import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:healthrian_common_support/healthrian_common_support.dart';
import 'package:screenshot/screenshot.dart';

const inputPath = 'test/input/60hz_noise.json';

enum Labels {
  l1,
  l2,
  l3,
  aVR,
  aVL,
  aVF,
  v1,
  v2,
  v3,
  v4,
  v5,
  v6,
}

class ChartPainter extends CustomPainter {
  ChartPainter(this.data);

  final List data;
  final _paint = Paint()..color = Colors.black;

  @override
  void paint(Canvas canvas, Size size) {
    if (data.length < 2) return;
    final w = size.width / (data.length - 1);
    final h = size.height / 6;
    for (var i = 0; i < data.length - 1; i++) {
      final double curr = data[i].toDouble();
      final double next = data[i + 1].toDouble();
      final currX = w * i;
      final nextX = w * (i + 1);
      final currY = ((curr > 3 ? 3 : (curr < -3 ? -3 : curr)) + 3) * h;
      final nextY = ((next > 3 ? 3 : (next < -3 ? -3 : next)) + 3) * h;

      canvas.drawLine(Offset(currX, currY), Offset(nextX, nextY), _paint);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return false;
  }
}

Future<void> main() async {
  await TestWidgetsFlutterBinding.ensureInitialized();
  await filter.waitForCompletion();

  // test('data_filter_unfiltered', () async {
  //   final dataFilter = DataFilter(1);
  //   dataFilter.dcBlock = DcBlock.off;
  //   dataFilter.bandStop = BandStop.off;
  //   dataFilter.highPass = HighPass.off;
  //   dataFilter.lowPass = LowPass.off;
  //   expect(dataFilter.applyFilter(0, 10), 10);
  // });

  test('data_filter_raw', () async {
    final dataFilter = DataFilter(12);
    dataFilter.impulseType = ImpulseType.iir;

    final input = File(inputPath);
    final inputAsString = input.readAsStringSync();
    final inputAsMap = jsonDecode(inputAsString);

    final filteredData = <String, List<double>>{};

    for (var i = 0; i < Labels.values.length; i++) {
      final label = Labels.values[i].name;

      filteredData[label] = [
        for (final d in inputAsMap[label])
          dataFilter.applyFilter(i, d.toDouble()),
      ];
    }

    final filteredDataAsString = jsonEncode(filteredData);

    final output = File('test/output/raw.json');
    await output.create(recursive: true);

    await output.writeAsString(filteredDataAsString);
  });

  test('data_filter_iir', () async {
    final dataFilter = DataFilter(12);
    dataFilter.impulseType = ImpulseType.iir;

    final input = File(inputPath);
    final inputAsString = input.readAsStringSync();
    final inputAsMap = jsonDecode(inputAsString);

    final filteredData = <String, List<double>>{};

    for (var i = 0; i < Labels.values.length; i++) {
      final label = Labels.values[i].name;

      filteredData[label] = [
        for (final d in inputAsMap[label])
          dataFilter.applyFilter(i, d.toDouble()),
      ];
    }

    final filteredDataAsString = jsonEncode(filteredData);

    final output = File('test/output/iir.json');
    await output.create(recursive: true);

    await output.writeAsString(filteredDataAsString);
  });

  test('data_filter_fir', () async {
    final dataFilter = DataFilter(12);
    dataFilter.impulseType = ImpulseType.fir;

    final input = File(inputPath);
    final inputAsString = input.readAsStringSync();
    final inputAsMap = jsonDecode(inputAsString);

    final filteredData = <String, List<double>>{};

    for (var i = 0; i < Labels.values.length; i++) {
      final label = Labels.values[i].name;

      filteredData[label] = [
        for (final d in inputAsMap[label])
          dataFilter.applyFilter(i, d.toDouble()),
      ];
    }

    final filteredDataAsString = jsonEncode(filteredData);

    final output = File('test/output/fir.json');
    await output.create(recursive: true);

    await output.writeAsString(filteredDataAsString);
  });

  test('data_filter_fir_zp', () async {
    final dataFilter = DataFilter(12);
    dataFilter.impulseType = ImpulseType.fir;

    final input = File(inputPath);
    final inputAsString = input.readAsStringSync();
    final inputAsMap = jsonDecode(inputAsString);
    final filteredData = <String, List<double>>{};

    for (var i = 0; i < Labels.values.length; i++) {
      final label = Labels.values[i].name;

      final delay = dataFilter.delaySample(i);
      filteredData[label] = [
        for (final d in inputAsMap[label])
          dataFilter.applyFilter(i, d.toDouble()),
        for (final d in inputAsMap[label]
            .sublist(
                inputAsMap[label].length - 1 - delay, inputAsMap[label].length)
            .reversed)
          dataFilter.applyFilter(i, d.toDouble()),
      ]..removeRange(0, delay);

      // final forward = [
      //   for (final d in inputAsMap[label])
      //     dataFilter.applyFilter(i, d.toDouble()),
      // ];
      //
      // final backward = [
      //   for (final d in forward.reversed)
      //     dataFilter.applyFilter(i, d.toDouble()),
      // ].reversed;
      // filteredData[label] = [...backward];
    }

    final filteredDataAsString = jsonEncode(filteredData);

    final output = File('test/output/fir_zp.json');
    await output.create(recursive: true);

    await output.writeAsString(filteredDataAsString);
  });

  test('data_filter_plot', () async {
    final jsonMap = {
      'raw': inputPath,
      'iir': 'test/output/iir.json',
      'fir': 'test/output/fir.json',
      'fir_zp': 'test/output/fir_zp.json',
    };
    for (final key in jsonMap.keys) {
      final input = File(jsonMap[key] ?? '');
      final inputAsString = await input.readAsString();
      final inputAsMap = jsonDecode(inputAsString);
      for (final label in Labels.values.map((e) => e.name)) {
        ScreenshotController screenshotController = ScreenshotController();
        await screenshotController
            .captureFromWidget(
          Container(
            color: Colors.white,
            width: inputAsMap[label].length / 50,
            height: 60,
            child: CustomPaint(
              painter: ChartPainter(inputAsMap[label]),
            ),
          ),
          pixelRatio: 10.0,
        )
            .then(
          (capturedImage) async {
            final outputPath = 'test/output/$key/$label.png';
            final output = File(outputPath);
            await output.create(recursive: true);
            await output.writeAsBytes(capturedImage);
            print(outputPath);
          },
        );
      }
    }
  });
}
